#include <fstream>
#include <iostream>
#include <string.h>
using namespace std;

const int MAXL = 50;
const int DIMCOD = 20;
const int DIMREP = 10; 

enum TipoRep {PEDIATRIA, ORTOPEDIA, MED_INTERNA, RADIOLOGIA, DA_ASSEGNARE};

struct Paziente {
  char nome[MAXL];
  char cognome[MAXL];
  TipoRep destinaz;
};

struct CodaPaz {
  Paziente paz[DIMCOD];
  int front, back; 
};

struct Reparto {
  Paziente paz[DIMREP];
  int numPaz;
};

struct Ospedale {
  CodaPaz accettazione;
  Reparto rep[RADIOLOGIA+1];
};


bool accetta(Ospedale* po, const char n[], const char c[], TipoRep r);
bool sposta(Ospedale* po);
bool trasferisci(Ospedale* po, const char n[], const char c[], TipoRep da, TipoRep a);
int quantiAffollato(const Ospedale* po, TipoRep* pq);
bool salva(const Ospedale* po, const char nf[]);

void init(Ospedale* po) {
  po->accettazione.front = 0;
  po->accettazione.back = 0;
  for(int i=0; i<RADIOLOGIA+1; i++)
    po->rep[i].numPaz = 0;
}

void stampaMenu(){
  cout << "-----------------------------" << endl;
  cout << "Menu:" << endl;
  cout << "0) stampa stato ospedale" << endl;
  cout << "1) accetta paziente" << endl;
  cout << "2) sposta paziente" << endl;
  cout << "3) trasferisci paziente" << endl;
  cout << "4) reparto piu' affollato" << endl;
  cout << "5) salva" << endl;
  cout << "6) esci" << endl;
  cout << "?" << endl;
}

TipoRep leggiTipoRep() {
  cout << "p) pediatria" << endl;
  cout << "o) ortopedia" << endl;
  cout << "m) medicina interna" << endl;
  cout << "r) radiologia" << endl;
  cout << "d) da assegnare" << endl;
  char c;
  while(true) {
    cin >> c;
    switch(c) {
      case 'p': case 'P':
        return PEDIATRIA;
      case 'o': case 'O':
        return ORTOPEDIA;
      case 'm': case 'M':
        return MED_INTERNA;
      case 'r': case 'R':
        return RADIOLOGIA;
      case 'd': case 'D':
        return DA_ASSEGNARE;
      default:
        cout << "Scelta non valida" << endl;
    }
  }
}

void provaAcc(Ospedale* po) {
  Paziente p;
  cout << "Inserisci nome paziente: ";
  cin >> p.nome;
  cout << "Inserisci cognome paziente: ";
  cin >> p.cognome;
  cout << "Inserisci il reparto di destinazione: " << endl;
  p.destinaz = leggiTipoRep();
  if(accetta(po, p.nome, p.cognome, p.destinaz)) cout << "Paziente accettato" << endl;
  else cout << "Paziente non accettato" << endl;
}

void provaTrasf(Ospedale* po) {
  Paziente p;
  cout << "Inserisci nome paziente: ";
  cin >> p.nome;
  cout << "Inserisci cognome paziente: ";
  cin >> p.cognome;
  cout << "Vecchio reparto:" << endl;
  TipoRep provenienza = leggiTipoRep();
  cout << "Nuovo reparto:" << endl;
  TipoRep destinazione = leggiTipoRep();
  if(trasferisci(po, p.nome, p.cognome, provenienza, destinazione)) cout << "Paziente trasferito" << endl;
  else cout << "Paziente non trasferito" << endl;
    
}

void stampaTipo(TipoRep tt) {
  switch(tt) {
    case PEDIATRIA:
      cout << "pediatria";
      break;
    case ORTOPEDIA:
      cout << "ortopedia";
      break;
    case MED_INTERNA:
      cout << "medicina interna";
      break;
    case RADIOLOGIA:
      cout << "radiologia";
      break;
    case DA_ASSEGNARE:
      cout << "da assegnare";
      break;
  }
}

void provaAff(const Ospedale* po) {
  TipoRep tt;
  int q;
  q = quantiAffollato(po, &tt);
  cout << "Il reparto piu' affollato contiene " << q << " pazienti ed ha tipo ";
  stampaTipo(tt); 
  cout << endl;
}

void provaSalva(const Ospedale* po){
  cout << "Salvo nel file elenco.txt" << endl;
  if(salva(po, "elenco.txt")) {
    cout << "Contenuto del file elenco.txt:" << endl;
    system("type elenco.txt");
  } else cout << "Salvataggio non riuscito" << endl;
}

void stampaPaz(Paziente p) {
  cout << p.cognome << ' ' << p.nome << ' ';
  stampaTipo(p.destinaz);
  cout << endl;
}

void stampaCod(CodaPaz cp) {
  for(int i=cp.front; i!=cp.back; i=(i+1)%DIMCOD)
    stampaPaz(cp.paz[i]);
}

void stampaRep(Reparto rr){
  for(int i=0; i<rr.numPaz; i++)
    stampaPaz(rr.paz[i]);
}

void stampaStato(const Ospedale* po) {
  cout << "Stato dell'ospedale:" << endl;
  cout << "accettazione:" << endl;
  stampaCod(po->accettazione);
  for(int i=PEDIATRIA; i<=RADIOLOGIA; i++) {
    stampaTipo((TipoRep)i);
    cout << ":" << endl;
    stampaRep(po->rep[i]);
  }
}

void provaSposta(Ospedale* po){
  if(sposta(po)) cout << "Paziente spostato nel reparto di destinazione" << endl;
  else cout << "Paziente rimesso in fondo alla coda (o nessun paziente presente in accettazione)" << endl;
}

int main(){
  Ospedale osp;
  init(&osp);

  char scelta;
  while(true) {
    stampaMenu();
    cin >> scelta;
    switch(scelta) {
      case '0':
        stampaStato(&osp);
        break;
      case '1':
        provaAcc(&osp);
        break;
      case '2':
        provaSposta(&osp);
        break;
      case '3':
        provaTrasf(&osp);
        break;
      case '4':
        provaAff(&osp);
        break;
      case '5':
        provaSalva(&osp);
        break;
      case '6':
        exit(0);
      default:
        cout << "Scelta non valida" << endl;
    }
  }
  return 0;
}
