#include <fstream>
#include <string.h>
using namespace std;

const int MAXL = 50;
const int DIMCOD = 20;
const int DIMREP = 10; 

enum TipoRep {PEDIATRIA, ORTOPEDIA, MED_INTERNA, RADIOLOGIA, DA_ASSEGNARE};

struct Paziente {
  char nome[MAXL];
  char cognome[MAXL];
  TipoRep destinaz;
};

struct CodaPaz {
  Paziente paz[DIMCOD];
  int front, back; 
};

struct Reparto {
  Paziente paz[DIMREP];
  int numPaz;
};

struct Ospedale {
  CodaPaz accettazione;
  Reparto rep[RADIOLOGIA+1];
};


bool codaFull(const CodaPaz* pc) {
  return (pc->back + 1) % DIMCOD == pc->front;
}

bool codaEmpty(const CodaPaz* pc) {
  return pc->back == pc->front;
}

bool insertCoda(CodaPaz* pc, Paziente v) {
  if(codaFull(pc)) return false;
  pc->paz[pc->back] = v;
  pc->back = (pc->back + 1) % DIMCOD;
  return true;
}

bool extractCoda(CodaPaz* pc, Paziente* pv) {
  if(codaEmpty(pc)) return false;
  *pv = pc->paz[pc->front];
  pc->front = (pc->front + 1) % DIMCOD;
  return true;
}

bool accetta(Ospedale* po, const char n[], const char c[], TipoRep r){
  Paziente p;
  strcpy(p.nome, n);
  strcpy(p.cognome, c);
  p.destinaz = r;
  return insertCoda(&po->accettazione, p);
}


bool sposta(Ospedale* po) {
  Paziente p;
  bool estratto = extractCoda(&po->accettazione, &p);
  if(!estratto) return false;
  if(p.destinaz == DA_ASSEGNARE ||  po->rep[p.destinaz].numPaz == DIMREP) {
    insertCoda(&po->accettazione, p);
    return false;
  } 
  po->rep[p.destinaz].paz[po->rep[p.destinaz].numPaz++] = p;
  return true;
}


bool trasferisci(Ospedale* po, const char n[], const char c[], TipoRep da, TipoRep a) {
  if(da == DA_ASSEGNARE || a == DA_ASSEGNARE) 
    return false;
  Reparto* p1;
  Reparto* p2;
  p1 = &po->rep[da];
  p2 = &po->rep[a];
  if(p2->numPaz == DIMREP) return false;
  for(int i=0; i<p1->numPaz; i++) {
    if(strcmp(n, p1->paz[i].nome) == 0 && strcmp(c, p1->paz[i].cognome) == 0) {
      Paziente tmp = p1->paz[i];
      tmp.destinaz = a;
      p2->paz[p2->numPaz++] = tmp;
      p1->numPaz--;
      p1->paz[i] = p1->paz[p1->numPaz];
      return true;
    }
  }
  return false;
}

int quantiAffollato(const Ospedale* po, TipoRep* pq) {
  int max = 0;
  for(int i=PEDIATRIA; i<=RADIOLOGIA; i++)
    if(po->rep[i].numPaz > max) {
      max = po->rep[i].numPaz;
      *pq = (TipoRep) i;
    }
  return max;
}

void scambia(Paziente v[], int x, int y) {
  Paziente tmp = v[x];
  v[x] = v[y];
  v[y] = tmp;
}

void ordina(Paziente v[], int n) {
  for(int i=0; i<n-1; i++) {
    int index = i;
    for(int j=i+1; j<n; j++)
      if(strcmp(v[j].cognome, v[index].cognome) < 0)
        index = j;
    scambia(v, index, i);
  }
}

bool salva(const Ospedale* po, const char nf[]) {
  Paziente aux[DIMCOD+DIMREP*(RADIOLOGIA-PEDIATRIA+1)];
  int conta = 0;
  
  for(int a=po->accettazione.front; a!=po->accettazione.back; a=(a+1)%DIMCOD)
    aux[conta++] = po->accettazione.paz[a];
  
  for(int i=PEDIATRIA; i<=RADIOLOGIA; i++)
    for(int j=0; j<po->rep[i].numPaz; j++)
      aux[conta++] = po->rep[i].paz[j];
  ordina(aux, conta);
  fstream f;
  f.open(nf, ios::out);
  if(!f) return false;
  for(int i=0; i<conta; i++)
    f << aux[i].cognome << " " << aux[i].nome << endl;
  return true;
}

